#! /usr/bin/env python
import random
import sys

if sys.version_info[0] >= 3:
	from scrib import barf, cfg, clean, version
else:
	import barf
	import cfg
	import clean
	import version

try:
	import redis
except:
	barf.Barf('ERR', 'Please install redis module for python.')
	sys.exit()


if sys.version_info[0] >= 3:
	xrange = range
	unicode = str
	basestring = str


class scrib:
	"""
	Setting up gatekeeping and processing.
	https://github.com/coleifer/irc/blob/master/bots/redisbot.py
	http://code.activestate.com/recipes/194364-the-markov-chain-algorithm/
	"""
	def __init__(self, interface, args):
		"""
		Here we'll load settings and set up modules.
		"""
		# Expose these for others to tap into
		self.barf = barf.Barf

		# Load scrib settings
		self.masterbrain = 'ataraxia.scndrls.net'
		self.masterport = 6379
		self.settings = self.setcfg()
		self.settings.load('scrib.cfg', {
			'host': self.masterbrain,
			'port': self.masterport,
			'debug': 0
		})
		self.version = version.__version__
		self.barf('MSG', 'scrib v%s initialized' % self.version)

		# Database vars
		self.host = self.settings.host
		self.port = self.settings.port
		self.tries = 0
		self.max_tries = 2

		self.redis = self.db_check(self.host, self.port)
		# Load interface settings
		self.interface = self.setcfg()
		self.interface.load(interface + '.cfg', args, suppress=True)
		self.prefix = 'scrib'  # Defaulting to universal brain
		self.name = self.name()

		# Set up us some internals
		self.messages_to_generate = 5
		self.separator = ' '
		
		self.clean = clean.Clean(self.settings.debug)

	def db_check(self, host='localhost', port='6379'):
		# Database check
		db = redis.Redis(host, port)
		# See if redis server is running
		try:
			ping = db.ping()
			if ping is not True:
				self.barf('ERR', 'Redis server is not running.')
				self.barf('ERR', 'Please install and run it, then try again.')
				sys.exit()
			else:
				self.barf('ACT', 'Connected to %s' % host)
				return db
		except Exception as e:
			self.barf('ERR', e.message)
			self.tries += 1

		while self.tries < self.max_tries:
			self.db_check(self.settings.host, self.settings.port)
		else:
			self.barf('ERR', 'No redis server found.')
			self.barf('ACT', 'Please make sure it is started or your host is correct.')
			sys.exit()

	"""
	Make some methods that give interfaces and plugins
	information in a controllable (and updatable) way.
	"""

	def setcfg(self):
		"""
		Simple wrapper for cfg.set()
		"""
		return cfg.set()

	def name(self):
		"""
		Simple wrapper for cfg.get_name()
		"""
		return cfg.get_name()

	def save_all(self):
		self.barf('SAV', 'Saving settings')
		self.settings.save()

	def shutdown(self):
		"""
		Shuts us the scrib down.
		"""
		msg = 'Shutting down. Goodbye!'
		try:
			if 'quitmsg' in self.interface._defaults:
				msg = self.interface._defaults['quitmsg']
		except:
			pass

		self.reply(msg, reply='', learn=False)
		self.save_all()
		self.barf('MSG', msg.replace(self.separator, ' '))
		sys.exit()

	def reply(self, message, reply='', learn=True):
		try:
			messages = []
			for words in self.clean.split(self.clean.sanitize(message)):
				key = self.separator.join(words[:-1])
				if learn is True:
					self.redis.sadd(self.make_key(key), words[-1])

				best_message = ''
				for i in range(self.messages_to_generate):
					generated = self.generate(seed=key)
					if len(generated) > len(best_message):
						best_message = generated

				if best_message:
					messages.append(best_message)

			if len(messages) and reply:
				if self.settings.debug is 1:
					self.barf('DBG', 'Replying')
				return random.choice(messages)
		except UnicodeDecodeError as e:
			# Don't need to crash every goddamn time, yo.
			self.barf('ERR', e)
			self.barf('ACT', 'Ignoring...')
			pass

	def make_key(self, k):
		return '-'.join((self.prefix, k))

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		if self.settings.debug is 1:
			self.barf('DBG', 'Analyzing...')
		key = seed
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)
			gen_words.append(words[0])
			next_word = self.redis.srandmember(self.make_key(key))

			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)
